==> org.robokind.impl.motion-0.8.5-sources.jar: org/robokind/impl/motion/messaging/PortableJointDefinition.java <==
/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.robokind.impl.motion.messaging;

import org.apache.avro.util.Utf8;
import org.robokind.api.common.position.NormalizedDouble;
import org.robokind.api.motion.Joint;
import org.robokind.api.motion.protocol.RobotDefinitionResponse.JointDefinition;
import org.robokind.avrogen.motion.JointDefinitionRecord;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class PortableJointDefinition implements JointDefinition{
    private JointDefinitionRecord myRecord;
    private Joint.Id myCachedId;
    private NormalizedDouble myCachedDefaultPosition;
    private NormalizedDouble myCachedGoalPosition;
    
    public PortableJointDefinition(JointDefinitionRecord record){
        if(record == null){
            throw new NullPointerException();
        }
        setRecord(record);
    }
    
    public PortableJointDefinition(JointDefinition jointDef){
        if(jointDef == null){
            throw new NullPointerException();
        }else if(jointDef instanceof PortableJointDefinition){
            PortableJointDefinition pr = ((PortableJointDefinition)jointDef);
            setRecord(pr.getRecord());
            return;
        }
        setRecord(jointDef.getJointId(), 
                jointDef.getName().toString(), 
                jointDef.getDefaultPosition(), 
                jointDef.getGoalPosition(), 
                jointDef.getEnabled());
    }
    
    public PortableJointDefinition(Joint.Id jId, String name, 
            NormalizedDouble defPos, NormalizedDouble goalPos, boolean enabled){
        setRecord(jId, name, defPos, goalPos, enabled);
    }
    
    public PortableJointDefinition(Joint joint){
        setRecord(joint.getId(), joint.getName(), 
                joint.getDefaultPosition(), 
                joint.getGoalPosition(), 
                joint.getEnabled());
    }
    
    private void setRecord(Joint.Id jId, String name, 
            NormalizedDouble defPos, NormalizedDouble goalPos, boolean enabled){
        JointDefinitionRecord rec = new JointDefinitionRecord();
        rec.jointId = jId.getLogicalJointNumber();
        rec.name = new Utf8(name);
        rec.defaultPosition = defPos.getValue();
        rec.goalPosition = goalPos.getValue();
        rec.enabled = enabled;
        myCachedId = jId;
        myCachedDefaultPosition = defPos;
        myCachedGoalPosition = goalPos;
        myRecord = rec;
    }
    
    private void setRecord(JointDefinitionRecord record){
        if(record == null){
            throw new NullPointerException();
        }
        myRecord = record;
        myCachedId = new Joint.Id(myRecord.jointId);
        myCachedDefaultPosition = 
                new NormalizedDouble(myRecord.defaultPosition);
        myCachedGoalPosition = new NormalizedDouble(myRecord.goalPosition);
    }
    
    @Override
    public Joint.Id getJointId() {
        return myCachedId;
    }

    @Override
    public String getName() {
        return myRecord.name.toString();
    }

    @Override
    public NormalizedDouble getDefaultPosition() {
        return myCachedDefaultPosition;
    }

    @Override
    public NormalizedDouble getGoalPosition() {
        return myCachedGoalPosition;
    }

    @Override
    public boolean getEnabled() {
        return myRecord.enabled;
    }
    
    public JointDefinitionRecord getRecord(){
        return myRecord;
    }
}

==> org.robokind.impl.motion-0.8.6-sources.jar: org/robokind/impl/motion/messaging/PortableJointDefinition.java <==
/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.robokind.impl.motion.messaging;

import java.util.ArrayList;
import java.util.List;
import org.apache.avro.Schema;
import org.apache.avro.generic.GenericData.Array;
import org.apache.avro.util.Utf8;
import org.robokind.api.common.position.NormalizedDouble;
import org.robokind.api.motion.Joint;
import org.robokind.api.motion.JointProperty;
import org.robokind.api.motion.protocol.RobotDefinitionResponse.JointDefinition;
import org.robokind.api.motion.protocol.RobotDefinitionResponse.JointPropDefinition;
import org.robokind.avrogen.motion.JointDefinitionRecord;
import org.robokind.avrogen.motion.JointPropDefinitionRecord;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class PortableJointDefinition implements JointDefinition{
    private JointDefinitionRecord myRecord;
    private Joint.Id myCachedId;
    private NormalizedDouble myCachedDefaultPosition;
    private NormalizedDouble myCachedGoalPosition;
    private List<JointPropDefinition> myPropDefs;
    
    public PortableJointDefinition(JointDefinitionRecord record){
        if(record == null){
            throw new NullPointerException();
        }
        setRecord(record);
        myPropDefs = new ArrayList<JointPropDefinition>(record.properties.size());
        for(JointPropDefinitionRecord prop : record.properties){
            myPropDefs.add(new PortableJointPropDefinition(prop));
        }
    }
    
    public PortableJointDefinition(JointDefinition jointDef){
        if(jointDef == null){
            throw new NullPointerException();
        }
        myPropDefs = jointDef.getJointProperties();
        if(jointDef instanceof PortableJointDefinition){
            PortableJointDefinition pr = ((PortableJointDefinition)jointDef);
            setRecord(pr.getRecord());
            return;
        }
        setRecord(jointDef.getJointId(), 
                jointDef.getName().toString(), 
                jointDef.getDefaultPosition(), 
                jointDef.getGoalPosition(), 
                jointDef.getEnabled(),
                jointDef.getJointProperties());
    }
    
    public PortableJointDefinition(Joint.Id jId, String name, 
            NormalizedDouble defPos, NormalizedDouble goalPos, boolean enabled,
            List<JointProperty> properties){
        myPropDefs = new ArrayList<JointPropDefinition>(properties.size());
        for(JointProperty prop : properties){
            PortableJointPropDefinition def = new PortableJointPropDefinition(prop);
            myPropDefs.add(def);
        }
        setRecord(jId, name, defPos, goalPos, enabled, myPropDefs);
    }
    
    public PortableJointDefinition(Joint joint){
        myPropDefs = new ArrayList<JointPropDefinition>(joint.getProperties().size());
        for(JointProperty prop : joint.getProperties()){
            PortableJointPropDefinition def = new PortableJointPropDefinition(prop);
            myPropDefs.add(def);
        }
        setRecord(joint.getId(), joint.getName(), 
                joint.getDefaultPosition(), 
                joint.getGoalPosition(), 
                joint.getEnabled(),
                myPropDefs);
    }
    
    private void setRecord(Joint.Id jId, String name, 
            NormalizedDouble defPos, NormalizedDouble goalPos, boolean enabled,
            List<JointPropDefinition> properties){
        JointDefinitionRecord rec = new JointDefinitionRecord();
        rec.jointId = jId.getLogicalJointNumber();
        rec.name = new Utf8(name);
        rec.defaultPosition = defPos.getValue();
        rec.goalPosition = goalPos.getValue();
        rec.enabled = enabled;
        rec.properties = 
                new Array<JointPropDefinitionRecord>(properties.size(), 
                        Schema.createArray(JointPropDefinitionRecord.SCHEMA$));
        myPropDefs = properties;
        for(JointPropDefinition prop : properties){
            rec.properties.add(new PortableJointPropDefinition(prop).getRecord());
        }
        myCachedId = jId;
        myCachedDefaultPosition = defPos;
        myCachedGoalPosition = goalPos;
        myRecord = rec;
    }
    
    private void setRecord(JointDefinitionRecord record){
        if(record == null){
            throw new NullPointerException();
        }
        myRecord = record;
        myCachedId = new Joint.Id(myRecord.jointId);
        myCachedDefaultPosition = 
                new NormalizedDouble(myRecord.defaultPosition);
        myCachedGoalPosition = new NormalizedDouble(myRecord.goalPosition);
    }
    
    @Override
    public Joint.Id getJointId() {
        return myCachedId;
    }

    @Override
    public String getName() {
        return myRecord.name.toString();
    }

    @Override
    public NormalizedDouble getDefaultPosition() {
        return myCachedDefaultPosition;
    }

    @Override
    public NormalizedDouble getGoalPosition() {
        return myCachedGoalPosition;
    }

    @Override
    public boolean getEnabled() {
        return myRecord.enabled;
    }

    @Override
    public List<JointPropDefinition> getJointProperties() {
        return myPropDefs;
    }
    
    public JointDefinitionRecord getRecord(){
        return myRecord;
    }
}
